==> polymer-core-0.9.19+1.21.1-sources.jar: eu/pb4/polymer/core/mixin/block/BlockStateMixin.java <==
package eu.pb4.polymer.core.mixin.block;

import com.llamalad7.mixinextras.injector.ModifyExpressionValue;
import com.mojang.serialization.Codec;
import eu.pb4.polymer.common.api.PolymerCommonUtils;
import eu.pb4.polymer.core.api.block.PolymerBlock;
import eu.pb4.polymer.core.api.block.PolymerBlockUtils;
import eu.pb4.polymer.core.api.entity.PolymerEntityUtils;
import eu.pb4.polymer.core.impl.interfaces.BlockStateExtra;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import xyz.nucleoid.packettweaker.PacketContext;

import java.util.function.Function;
import net.minecraft.class_2680;

@Mixin(class_2680.class)
public abstract class BlockStateMixin implements BlockStateExtra {
    @Shadow protected abstract class_2680 asBlockState();

    @Unique
    private boolean polymer$calculatedIsLight;
    @Unique
    private boolean polymer$isLight;

    @Override
    public boolean polymer$isPolymerLightSource() {
        if (this.polymer$calculatedIsLight) {
            return this.polymer$isLight;
        }

        if (this.asBlockState().method_26204() instanceof PolymerBlock polymerBlock) {
            this.polymer$isLight = this.asBlockState().method_26213() != polymerBlock.getPolymerBlockState(this.asBlockState()).method_26213();
        }


        this.polymer$calculatedIsLight = true;

        return false;
    }

    @ModifyExpressionValue(method = "<clinit>", at = @At(value = "INVOKE", target = "Lnet/minecraft/block/BlockState;createCodec(Lcom/mojang/serialization/Codec;Ljava/util/function/Function;)Lcom/mojang/serialization/Codec;"))
    private static Codec<class_2680> patchCodec(Codec<class_2680> codec) {
        return codec.xmap(Function.identity(), content -> { // Encode
            if (PolymerCommonUtils.isServerNetworkingThreadWithContext()  && content.method_26204() instanceof PolymerBlock) {
                return PolymerBlockUtils.getPolymerBlockState(content);
            }
            return content;
        });
    }
}

==> polymer-core-0.14.0-rc.1+1.21.9-rc1-sources.jar: eu/pb4/polymer/core/mixin/block/BlockStateMixin.java <==
package eu.pb4.polymer.core.mixin.block;

import com.llamalad7.mixinextras.injector.ModifyExpressionValue;
import com.mojang.serialization.Codec;
import eu.pb4.polymer.common.api.PolymerCommonUtils;
import eu.pb4.polymer.core.api.block.PolymerBlock;
import eu.pb4.polymer.core.api.block.PolymerBlockUtils;
import eu.pb4.polymer.core.api.utils.PolymerSyncedObject;
import eu.pb4.polymer.core.impl.interfaces.BlockStateExtra;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import xyz.nucleoid.packettweaker.PacketContext;

import java.util.function.Function;
import net.minecraft.class_2680;
import net.minecraft.class_7923;

@Mixin(class_2680.class)
public abstract class BlockStateMixin implements BlockStateExtra {
    @Shadow protected abstract class_2680 asBlockState();

    @Unique
    private boolean polymer$calculatedIsLight;
    @Unique
    private boolean polymer$isLight;

    @Override
    public boolean polymer$isPolymerLightSource() {
        if (this.polymer$calculatedIsLight) {
            return this.polymer$isLight;
        }

        if (PolymerSyncedObject.getSyncedObject(class_7923.field_41175, this.asBlockState().method_26204()) instanceof PolymerBlock polymerBlock) {
            this.polymer$isLight = this.asBlockState().method_26213() != polymerBlock.getPolymerBlockState(this.asBlockState(), PacketContext.create()).method_26213();
        }


        this.polymer$calculatedIsLight = true;
        return false;
    }

    @ModifyExpressionValue(method = "<clinit>", at = @At(value = "INVOKE", target = "Lnet/minecraft/block/BlockState;createCodec(Lcom/mojang/serialization/Codec;Ljava/util/function/Function;)Lcom/mojang/serialization/Codec;"))
    private static Codec<class_2680> patchCodec(Codec<class_2680> codec) {
        return codec.xmap(Function.identity(), content -> { // Encode
            if (PolymerCommonUtils.isServerNetworkingThreadWithContext() && PolymerSyncedObject.getSyncedObject(class_7923.field_41175, content.method_26204()) != null) {
                return PolymerBlockUtils.getPolymerBlockState(content, PacketContext.get());
            }
            return content;
        });
    }
}
